@@ -364,7 +364,7 @@ def field(
     obj: _X,
     *,
     name: str = "",
-    widget_type: str | type[WidgetProtocol] | None = None,
+    widget_type: str | type[WidgetProtocol] | type[Widget] | None = None,
     options: dict[str, Any] = {},
     record: bool = True,
 ) -> MagicField[ValueWidget, _X]:
@@ -376,7 +376,7 @@ def field(
     obj: type[_W],
     *,
     name: str = "",
-    widget_type: str | type[WidgetProtocol] | None = None,
+    widget_type: str | type[WidgetProtocol] | type[Widget] | None = None,
     options: dict[str, Any] = {},
     record: bool = True,
 ) -> MagicField[_W, Any]:
@@ -388,7 +388,7 @@ def field(
     obj: type[_X],
     *,
     name: str = "",
-    widget_type: str | type[WidgetProtocol] | None = None,
+    widget_type: str | type[WidgetProtocol] | type[Widget] | None = None,
     options: dict[str, Any] = {},
     record: bool = True,
 ) -> MagicField[ValueWidget, _X]:
@@ -400,13 +400,25 @@ def field(
     obj: type[_M],
     *,
     name: str = "",
-    widget_type: str | type[WidgetProtocol] | None = None,
+    widget_type: str | type[WidgetProtocol] | type[Widget] | None = None,
     options: dict[str, Any] = {},
     record: bool = True,
 ) -> MagicField[_M, Any]:
     ...
 
 
+@overload
+def field(
+    obj: Any,
+    *,
+    name: str = "",
+    widget_type: type[_W] = None,
+    options: dict[str, Any] = {},
+    record: bool = True,
+) -> MagicField[_W, Any]:
+    ...
+
+
 @overload
 def field(
     obj: Any,
@@ -460,7 +472,7 @@ def vfield(
     obj: _X,
     *,
     name: str = "",
-    widget_type: str | type[WidgetProtocol] | None = None,
+    widget_type: str | type[WidgetProtocol] | type[Widget] | None = None,
     options: dict[str, Any] = {},
     record: bool = True,
 ) -> MagicValueField[ValueWidget, _X]:
@@ -472,7 +484,7 @@ def vfield(
     obj: type[_W],
     *,
     name: str = "",
-    widget_type: str | type[WidgetProtocol] | None = None,
+    widget_type: str | type[WidgetProtocol] | type[Widget] | None = None,
     options: dict[str, Any] = {},
     record: bool = True,
 ) -> MagicValueField[_W, Any]:
@@ -484,7 +496,7 @@ def vfield(
     obj: type[_X],
     *,
     name: str = "",
-    widget_type: str | type[WidgetProtocol] | None = None,
+    widget_type: str | type[WidgetProtocol] | type[Widget] | None = None,
     options: dict[str, Any] = {},
     record: bool = True,
 ) -> MagicValueField[ValueWidget, _X]:
@@ -496,7 +508,19 @@ def vfield(
     obj: Any,
     *,
     name: str = "",
-    widget_type: str | type[WidgetProtocol] | None = None,
+    widget_type: type[_W] = None,
+    options: dict[str, Any] = {},
+    record: bool = True,
+) -> MagicValueField[_W, Any]:
+    ...
+
+
+@overload
+def vfield(
+    obj: Any,
+    *,
+    name: str = "",
+    widget_type: str | type[WidgetProtocol] | type[Widget] | None = None,
     options: dict[str, Any] = {},
     record: bool = True,
 ) -> MagicValueField[Widget, Any]:
@@ -555,7 +579,8 @@ def _get_field(
         raise TypeError(f"Field options must be a dict, got {type(options)}")
     options = options.copy()
     metadata = dict(widget_type=widget_type, options=options)
-
+    if "name" in options.keys():
+        name = options["name"]
     kwargs = dict(metadata=metadata, name=name, record=record)
     if isinstance(obj, type):
         f = field_class(default_factory=obj, **kwargs)
