@@ -11,13 +11,13 @@ from typing import (
     MutableSequence,
 )
 from types import MethodType
-from typing_extensions import _AnnotatedAlias
+from abc import ABCMeta
+from typing_extensions import _AnnotatedAlias, Literal
 import inspect
 import warnings
 import os
 from enum import Enum
 import warnings
-import weakref
 from docstring_parser import parse, compose
 from qtpy.QtWidgets import QWidget, QDockWidget
 from qtpy.QtGui import QIcon
@@ -49,10 +49,16 @@ from .mgui_ext import (
     _LabeledWidgetAction,
     mguiLike,
 )
-from .utils import get_parameters, define_callback
+from .utils import get_parameters, define_callback, callable_to_classes
 from ._macro import GuiMacro
 
-from ..utils import get_signature, iter_members, extract_tooltip, screen_center
+from ..utils import (
+    get_signature,
+    iter_members,
+    extract_tooltip,
+    move_to_screen_center,
+    thread_worker,
+)
 from ..widgets import Separator, FreeWidget
 from ..fields import MagicField
 from ..signature import MagicMethodSignature, get_additional_option
@@ -73,11 +79,56 @@ class PopUpMode(Enum):
     parentlast = "parentlast"
 
 
+def _msgbox_raising(e, parent):
+    from ._message_box import QtErrorMessageBox
+
+    return QtErrorMessageBox.raise_(e, parent=parent.native)
+
+
+def _stderr_raising(e, parent):
+    pass
+
+
+def _stdout_raising(e, parent):
+    print(f"{e.__class__.__name__}: {e}")
+
+
 class ErrorMode(Enum):
     msgbox = "msgbox"
     stderr = "stderr"
     stdout = "stdout"
 
+    def get_handler(self):
+        """Get error handler."""
+        return ErrorModeHandlers[self]
+
+    def wrap_handler(self, func: Callable, parent):
+        """Wrap function with the error handler."""
+        handler = self.get_handler()
+
+        def wrapped_func(*args, **kwargs):
+            try:
+                out = func(*args, **kwargs)
+            except Exception as e:
+                handler(e, parent=parent)
+                out = e
+            return out
+
+        wrapped_func.__annotations__ = func.__annotations__
+        # wrapped_func.__name__ = func.__name__  # this is not allowed!
+        wrapped_func.__qualname__ = func.__qualname__
+        wrapped_func.__doc__ = func.__doc__
+        wrapped_func.__module__ = func.__module__
+
+        return wrapped_func
+
+
+ErrorModeHandlers = {
+    ErrorMode.msgbox: _msgbox_raising,
+    ErrorMode.stderr: _stderr_raising,
+    ErrorMode.stdout: _stdout_raising,
+}
+
 
 defaults = {
     "popup_mode": PopUpMode.popup,
@@ -148,15 +199,32 @@ def check_override(cls: type):
 
 
 _T = TypeVar("_T", bound="MagicTemplate")
+_F = TypeVar("_F", bound=Callable)
 
 
-class MagicTemplate:
+class _MagicTemplateMeta(ABCMeta):
+    """This metaclass enables type checking of nested magicclasses."""
+
+    @overload
+    def __get__(self: type[_T], obj: Any, objtype=None) -> _T:
+        ...
+
+    @overload
+    def __get__(self: type[_T], obj: Literal[None], objtype=None) -> type[_T]:
+        ...
+
+    def __get__(self, obj, objtype=None):
+        return self
+
+
+class MagicTemplate(metaclass=_MagicTemplateMeta):
     __doc__ = ""
     __magicclass_parent__: None | MagicTemplate
     __magicclass_children__: list[MagicTemplate]
     _close_on_run: bool
     _component_class: type[Action | Widget]
     _error_mode: ErrorMode
+    _list: list[Action | Widget]
     _macro_instance: GuiMacro
     _my_symbol: Symbol
     _popup_mode: PopUpMode
@@ -235,6 +303,7 @@ class MagicTemplate:
 
     @property
     def macro(self) -> GuiMacro:
+        """The macro object bound to the ancestor widget."""
         if self.__magicclass_parent__ is None:
             return self._macro_instance
         else:
@@ -242,9 +311,7 @@ class MagicTemplate:
 
     @property
     def parent_viewer(self) -> napari.Viewer | None:
-        """
-        Return napari.Viewer if magic class is a dock widget of a viewer.
-        """
+        """Return napari.Viewer if magic class is a dock widget of a viewer."""
         parent_self = self._search_parent_magicclass()
         if parent_self.native.parent() is None:
             return None
@@ -321,11 +388,11 @@ class MagicTemplate:
     @classmethod
     def wraps(
         cls,
-        method: Callable | None = None,
+        method: _F | None = None,
         *,
         template: Callable | None = None,
         copy: bool = False,
-    ) -> Callable:
+    ) -> _F:
         """
         Wrap a parent method in a child magic-class.
 
@@ -360,7 +427,7 @@ class MagicTemplate:
             # If method is already wrapped, wraps should create a copy.
             copy = True
 
-        def wrapper(method: Callable):
+        def wrapper(method: _F):
             # Base function to get access to the original function
             if isinstance(method, FunctionGui):
                 func = method._function
@@ -514,17 +581,7 @@ class MagicTemplate:
         widget = self._component_class(name=obj.__name__, text=text, gui_only=True)
 
         # Wrap function to deal with errors in a right way.
-        if self._error_mode == ErrorMode.msgbox:
-            wrapper = _raise_error_in_msgbox
-        elif self._error_mode == ErrorMode.stderr:
-            wrapper = _identity_wrapper
-        elif self._error_mode == ErrorMode.stdout:
-            wrapper = _print_error
-        else:
-            raise ValueError(self._error_mode)
-
-        # Wrap function to block macro recording.
-        func = functools_wraps(obj)(wrapper(obj, parent=self))
+        func = self._error_mode.wrap_handler(obj, parent=self)
 
         # Signature must be updated like this. Otherwise, already wrapped member function
         # will have signature with "self".
@@ -572,8 +629,10 @@ class MagicTemplate:
         func.__signature__ = func.__signature__.replace(parameters=all_params)
         # Get the number of parameters except for empty widgets.
         # With these lines, "bind" method of magicgui works inside magicclass.
-        fgui = FunctionGuiPlus.from_callable(func)
-        n_empty = len([_widget for _widget in fgui if isinstance(_widget, EmptyWidget)])
+        fgui_classes = callable_to_classes(func)
+        n_empty = len(
+            [_wdg_cls for _wdg_cls in fgui_classes if _wdg_cls is EmptyWidget]
+        )
         nparams = _n_parameters(func) - n_empty
 
         if isinstance(func.__signature__, MagicMethodSignature):
@@ -581,7 +640,9 @@ class MagicTemplate:
                 obj_sig, "additional_options", {}
             )
 
-        if nparams == 0:
+        has_preview = get_additional_option(func, "preview", None) is not None
+
+        if nparams == 0 and not has_preview:
             # We don't want a dialog with a single widget "Run" to show up.
             def run_function():
                 # NOTE: callback must be defined inside function. Magic class must be
@@ -592,20 +653,18 @@ class MagicTemplate:
 
                 return out
 
-        elif nparams == 1 and isinstance(fgui[0], FileEdit):
+        elif nparams == 1 and issubclass(fgui_classes[0], FileEdit) and not has_preview:
             # We don't want to open a magicgui dialog and again open a file dialog.
             def run_function():
                 mgui = _build_mgui(widget, func, self)
                 mgui.native.setParent(self.native, mgui.native.windowFlags())
                 fdialog: FileEdit = mgui[0]
-                result = fdialog._show_file_dialog(
+                if result := fdialog._show_file_dialog(
                     fdialog.mode,
                     caption=fdialog._btn_text,
                     start_path=str(fdialog.value),
                     filter=fdialog.filter,
-                )
-
-                if result:
+                ):
                     fdialog.value = result
                     out = mgui(result)
                 else:
@@ -613,12 +672,12 @@ class MagicTemplate:
                 return out
 
         else:
+            _prep_func = _define_popup(self, obj, widget)
 
             def run_function():
                 mgui = _build_mgui(widget, func, self)
                 if mgui.call_count == 0 and len(mgui.called._slots) == 0:
-
-                    # deal with popup mode.
+                    _prep_func(mgui)
                     if self._popup_mode not in (PopUpMode.popup, PopUpMode.dock):
                         mgui.label = ""
                         # to avoid name collision
@@ -631,52 +690,6 @@ class MagicTemplate:
                         # TODO: should remove mgui from self?
                         title.btn_clicked.connect(mgui.hide)
                         mgui.insert(0, title)
-                        mgui.append(Separator(orientation="horizontal"))
-
-                        if self._popup_mode == PopUpMode.parentlast:
-                            parent_self = self._search_parent_magicclass()
-                            parent_self.append(mgui)
-                        elif self._popup_mode == PopUpMode.first:
-                            child_self = _child_that_has_widget(self, obj, widget)
-                            child_self.insert(0, mgui)
-                        elif self._popup_mode == PopUpMode.last:
-                            child_self = _child_that_has_widget(self, obj, widget)
-                            child_self.append(mgui)
-                        elif self._popup_mode == PopUpMode.above:
-                            child_self = _child_that_has_widget(self, obj, widget)
-                            i = _get_index(child_self, widget)
-                            child_self.insert(i, mgui)
-                        elif self._popup_mode == PopUpMode.below:
-                            child_self = _child_that_has_widget(self, obj, widget)
-                            i = _get_index(child_self, widget)
-                            child_self.insert(i + 1, mgui)
-
-                    elif self._popup_mode == PopUpMode.dock:
-                        from .class_gui import MainWindowClassGui
-
-                        parent_self = self._search_parent_magicclass()
-                        viewer = parent_self.parent_viewer
-                        if viewer is None:
-                            if isinstance(parent_self, MainWindowClassGui):
-                                parent_self.add_dock_widget(mgui)
-                            else:
-                                msg = (
-                                    "Cannot add dock widget to a normal container. Please use\n"
-                                    ">>> @magicclass(widget_type='mainwindow')\n"
-                                    "to create main window widget, or add the container as a dock "
-                                    "widget in napari."
-                                )
-                                warnings.warn(msg, UserWarning)
-
-                        else:
-                            viewer.window.add_dock_widget(
-                                mgui, name=_get_widget_name(widget), area="right"
-                            )
-                    else:
-                        # To be popped up correctly, window flags of FunctionGui should be
-                        # "windowFlags" and should appear at the center.
-                        mgui.native.setParent(self.native, mgui.native.windowFlags())
-                        mgui.native.move(screen_center() - mgui.native.rect().center())
 
                     if self._close_on_run and not mgui._auto_call:
                         if self._popup_mode != PopUpMode.dock:
@@ -685,11 +698,22 @@ class MagicTemplate:
                             # If FunctioGui is docked, we should close QDockWidget.
                             mgui.called.connect(lambda: mgui.parent.hide())
 
+                if nparams == 1 and issubclass(fgui_classes[0], FileEdit):
+                    fdialog: FileEdit = mgui[0]
+                    if result := fdialog._show_file_dialog(
+                        fdialog.mode,
+                        caption=fdialog._btn_text,
+                        start_path=str(fdialog.value),
+                        filter=fdialog.filter,
+                    ):
+                        fdialog.value = result
+                    else:
+                        return None
+
                 if self._popup_mode != PopUpMode.dock:
                     widget.mgui.show()
                 else:
-                    # show dock widget
-                    mgui.parent.show()
+                    mgui.parent.show()  # show dock widget
 
                 return None
 
@@ -701,14 +725,8 @@ class MagicTemplate:
         # keybinding
         keybinding = get_additional_option(func, "keybinding", None)
         if keybinding is not None:
-            if obj.__name__.startswith("_"):
-                from qtpy.QtWidgets import QShortcut
-
-                shortcut = QShortcut(as_shortcut(keybinding), self.native)
-                shortcut.activated.connect(widget.changed)
-            else:
-                shortcut = as_shortcut(keybinding)
-                widget.set_shortcut(shortcut)
+            shortcut = as_shortcut(keybinding)
+            widget.set_shortcut(shortcut)
 
         return widget
 
@@ -750,15 +768,6 @@ class ContainerLikeGui(BaseGui, mguiLike, MutableSequence):
     changed = Signal(object)
     _list: list[AbstractAction | ContainerLikeGui]
 
-    def reset_choices(self, *_: Any):
-        """Reset child Categorical widgets"""
-        for widget in self:
-            if hasattr(widget, "reset_choices"):
-                widget.reset_choices()
-        for widget in self.__magicclass_children__:
-            if hasattr(widget, "reset_choices"):
-                widget.reset_choices()
-
     @property
     def icon_path(self):
         return self._icon_path
@@ -778,6 +787,20 @@ class ContainerLikeGui(BaseGui, mguiLike, MutableSequence):
                 f"Path {path} does not exists. Could not set icon.", UserWarning
             )
 
+    def reset_choices(self, *_: Any):
+        """Reset child Categorical widgets"""
+        all_widgets: set[Widget] = set()
+
+        for item in self._list:
+            widget = getattr(item, "_inner_widget", item)
+            all_widgets.add(widget)
+        for widget in self.__magicclass_children__:
+            all_widgets.add(widget)
+
+        for w in all_widgets:
+            if hasattr(w, "reset_choices"):
+                w.reset_choices()
+
     def _create_widget_from_field(self, name: str, fld: MagicField):
         if fld.not_ready():
             raise TypeError(
@@ -912,6 +935,11 @@ def _build_mgui(widget_: Action | PushButtonPlus, func: Callable, parent: BaseGu
         mgui = FunctionGuiPlus(
             func, call_button, layout=layout, labels=labels, auto_call=auto_call
         )
+        preview = opt.get("preview", None)
+        if preview is not None:
+            btn_text, previewer = preview
+            mgui.append_preview(previewer.__get__(parent), btn_text)
+
     except Exception as e:
         msg = (
             "Exception was raised during building magicgui from method "
@@ -922,7 +950,6 @@ def _build_mgui(widget_: Action | PushButtonPlus, func: Callable, parent: BaseGu
     widget_.mgui = mgui
     name = widget_.name or ""
     mgui.native.setWindowTitle(name.replace("_", " ").strip())
-    mgui._magicclass_parent_ref = weakref.ref(parent)
     return mgui
 
 
@@ -1002,7 +1029,7 @@ def wraps(template: Callable | inspect.Signature) -> Callable[[_C], _C]:
     return wrapper
 
 
-def _raise_error_in_msgbox(_func: Callable, parent: Widget = None):
+def _raise_error_in_msgbox(_func: Callable, parent: BaseGui = None):
     """If exception happened inside function, then open a message box."""
 
     def wrapped_func(*args, **kwargs):
@@ -1018,7 +1045,7 @@ def _raise_error_in_msgbox(_func: Callable, parent: Widget = None):
     return wrapped_func
 
 
-def _print_error(_func: Callable, parent: Widget = None):
+def _print_error(_func: Callable, parent: BaseGui = None):
     """If exception happened inside function, then print it."""
 
     def wrapped_func(*args, **kwargs):
@@ -1032,7 +1059,7 @@ def _print_error(_func: Callable, parent: Widget = None):
     return wrapped_func
 
 
-def _identity_wrapper(_func: Callable, parent: Widget = None):
+def _identity_wrapper(_func: Callable, parent: BaseGui = None):
     """Do nothing."""
 
     def wrapped_func(*args, **kwargs):
@@ -1217,6 +1244,9 @@ def nested_function_gui_callback(gui: MagicTemplate, fgui: FunctionGui):
     return _after_run
 
 
+_SELF = inspect.Parameter("self", inspect.Parameter.POSITIONAL_OR_KEYWORD)
+
+
 def _inject_recorder(func: Callable, is_method: bool = True) -> Callable:
     """Inject macro recording functionality into a function."""
     sig = get_signature(func)
@@ -1232,9 +1262,8 @@ def _inject_recorder(func: Callable, is_method: bool = True) -> Callable:
         def _func(self, *args, **kwargs):
             return func(*args, **kwargs)
 
-        _self_param = inspect.Parameter("self", inspect.Parameter.POSITIONAL_OR_KEYWORD)
         _func.__signature__ = sig.replace(
-            parameters=[_self_param] + list(sig.parameters.values()),
+            parameters=[_SELF] + list(sig.parameters.values()),
             return_annotation=sig.return_annotation,
         )
 
@@ -1243,20 +1272,15 @@ def _inject_recorder(func: Callable, is_method: bool = True) -> Callable:
         _auto_call = opt.get("auto_call", False)
     else:
         _auto_call = False
+
     # TODO: if function has a return_annotation, macro should be recorded like ui["f"](...)
-    @functools_wraps(_func)
-    def _recordable(bgui: MagicTemplate, *args, **kwargs):
-        with bgui.macro.blocked():
-            out = _func.__get__(bgui)(*args, **kwargs)
-        if not bgui.macro.active:
-            return out
+    def _record_macro(bgui: MagicTemplate, *args, **kwargs):
         bound = sig.bind(*args, **kwargs)
         kwargs = dict(bound.arguments.items())
         expr = Expr.parse_method(bgui, _func, (), kwargs)
-
         if _auto_call:
-            # Auto-call will cause many redundant macros. To avoid this, only the last input
-            # will be recorded in magic-class.
+            # Auto-call will cause many redundant macros. To avoid this, only the last
+            # input will be recorded in magic-class.
             last_expr = bgui.macro[-1]
             if (
                 last_expr.head == Head.call
@@ -1269,11 +1293,39 @@ def _inject_recorder(func: Callable, is_method: bool = True) -> Callable:
 
         bgui.macro.append(expr)
         bgui.macro._last_setval = None
-        return out
+        return None
 
-    if hasattr(_func, "__signature__"):
-        _recordable.__signature__ = _func.__signature__
-    return _recordable
+    if not isinstance(_func, thread_worker):
+
+        @functools_wraps(_func)
+        def _recordable(bgui: MagicTemplate, *args, **kwargs):
+            with bgui.macro.blocked():
+                out = _func.__get__(bgui)(*args, **kwargs)
+            if bgui.macro.active:
+                _record_macro(bgui, *args, **kwargs)
+            return out
+
+        if hasattr(_func, "__signature__"):
+            _recordable.__signature__ = _func.__signature__
+        return _recordable
+
+    else:
+        _func.returned.connect(_define_returned_callback(_func, _record_macro))
+        return _func
+
+
+def _define_returned_callback(worker: thread_worker, recorder):
+    """
+    Define a returned callback function for macro-recording of a
+    thread_worker object.
+    """
+
+    def _on_return(bgui: MagicTemplate, _=None):
+        args, kwargs = worker._last_arguments
+        recorder(bgui, *args, **kwargs)
+        return None
+
+    return _on_return
 
 
 def convert_attributes(cls: type[_T], hide: tuple[type, ...]) -> dict[str, Any]:
@@ -1303,6 +1355,7 @@ def convert_attributes(cls: type[_T], hide: tuple[type, ...]) -> dict[str, Any]:
     for subcls in reversed(mro):
         for name, obj in subcls.__dict__.items():
             if name.startswith("_") or isinstance(obj, _pass_type) or not callable(obj):
+                # private method, non-action-like object, not-callable object are passed.
                 new_attr = obj
             elif callable(obj) and get_additional_option(obj, "record", True):
                 new_attr = _inject_recorder(obj)
@@ -1311,3 +1364,73 @@ def convert_attributes(cls: type[_T], hide: tuple[type, ...]) -> dict[str, Any]:
 
             _dict[name] = new_attr
     return _dict
+
+
+def _define_popup(self: BaseGui, obj, widget: PushButtonPlus | Action):
+    # deal with popup mode.
+    if self._popup_mode == PopUpMode.popup:
+        # To be popped up correctly, window flags of FunctionGui should be
+        # "windowFlags" and should appear at the center.
+        def _prep(mgui: FunctionGui):
+            mgui.native.setParent(self.native, mgui.native.windowFlags())
+            move_to_screen_center(mgui.native)
+
+    elif self._popup_mode == PopUpMode.parentlast:
+
+        def _prep(mgui: FunctionGui):
+            parent_self = self._search_parent_magicclass()
+            parent_self.append(mgui)
+
+    elif self._popup_mode == PopUpMode.first:
+
+        def _prep(mgui: FunctionGui):
+            child_self = _child_that_has_widget(self, obj, widget)
+            child_self.insert(0, mgui)
+
+    elif self._popup_mode == PopUpMode.last:
+
+        def _prep(mgui: FunctionGui):
+            child_self = _child_that_has_widget(self, obj, widget)
+            child_self.append(mgui)
+
+    elif self._popup_mode == PopUpMode.above:
+
+        def _prep(mgui: FunctionGui):
+            child_self = _child_that_has_widget(self, obj, widget)
+            i = _get_index(child_self, widget)
+            child_self.insert(i, mgui)
+
+    elif self._popup_mode == PopUpMode.below:
+
+        def _prep(mgui: FunctionGui):
+            child_self = _child_that_has_widget(self, obj, widget)
+            i = _get_index(child_self, widget)
+            child_self.insert(i + 1, mgui)
+
+    elif self._popup_mode == PopUpMode.dock:
+        from .class_gui import MainWindowClassGui
+
+        def _prep(mgui: FunctionGui):
+
+            parent_self = self._search_parent_magicclass()
+            viewer = parent_self.parent_viewer
+            if viewer is None:
+                if isinstance(parent_self, MainWindowClassGui):
+                    parent_self.add_dock_widget(mgui)
+                else:
+                    msg = (
+                        "Cannot add dock widget to a normal container. Please use\n"
+                        ">>> @magicclass(widget_type='mainwindow')\n"
+                        "to create main window widget, or add the container as a dock "
+                        "widget in napari."
+                    )
+                    warnings.warn(msg, UserWarning)
+
+            else:
+                viewer.window.add_dock_widget(
+                    mgui, name=_get_widget_name(widget), area="right"
+                )
+
+    else:
+        raise RuntimeError
+    return _prep
