@@ -2,13 +2,15 @@ from __future__ import annotations
 from inspect import signature
 from typing import Any, Callable, Sequence, TypeVar
 import warnings
-from qtpy.QtWidgets import QMenuBar, QWidget, QMainWindow, QBoxLayout
+from qtpy.QtWidgets import QMenuBar, QWidget, QMainWindow, QBoxLayout, QDockWidget
 from qtpy.QtCore import Qt
 from magicgui.widgets import Container, MainWindow, Label, FunctionGui, Image, Table
 from magicgui.widgets._bases import Widget, ButtonWidget, ValueWidget, ContainerWidget
 from magicgui.widgets._concrete import _LabeledWidget, ContainerWidget
 from macrokit import Symbol
 
+
+from .keybinding import as_shortcut
 from .mgui_ext import PushButtonPlus
 from .toolbar import ToolBarGui, QtTabToolBar
 from .menu_gui import MenuGui, ContextMenuGui
@@ -18,12 +20,11 @@ from ._base import (
     ErrorMode,
     value_widget_callback,
     nested_function_gui_callback,
-    _inject_recorder,
 )
 from .utils import (
     copy_class,
     define_callback,
-    MagicClassConstructionError,
+    format_error,
     set_context_menu,
 )
 from ..widgets import (
@@ -47,7 +48,6 @@ from ..utils import iter_members, extract_tooltip
 from ..fields import MagicField
 from ..signature import get_additional_option
 from .._app import run_app
-from ._dock_widget import QtDockWidget
 
 # For Containers that belong to these classes, menubar must be set to _qwidget.layout().
 _USE_OUTER_LAYOUT = (
@@ -57,6 +57,8 @@ _USE_OUTER_LAYOUT = (
     TabbedContainer,
 )
 
+_MCLS_PAREMT = "__magicclass_parent__"
+
 
 class ClassGuiBase(BaseGui):
     # This class is always inherited by @magicclass decorator.
@@ -226,6 +228,14 @@ class ClassGuiBase(BaseGui):
                         if name.startswith("_") or not get_additional_option(
                             attr, "gui", True
                         ):
+                            keybinding = get_additional_option(attr, "keybinding", None)
+                            if keybinding:
+                                from qtpy.QtWidgets import QShortcut
+
+                                shortcut = QShortcut(
+                                    as_shortcut(keybinding), self.native
+                                )
+                                shortcut.activated.connect(widget)
                             continue
                         try:
                             widget = self._create_widget_from_method(widget)
@@ -237,8 +247,8 @@ class ClassGuiBase(BaseGui):
                             )
                             continue
 
-                    elif hasattr(widget, "__magicclass_parent__") or hasattr(
-                        widget.__class__, "__magicclass_parent__"
+                    elif hasattr(widget, _MCLS_PAREMT) or hasattr(
+                        widget.__class__, _MCLS_PAREMT
                     ):
                         # magic-class has to know its parent.
                         # if __magicclass_parent__ is defined as a property, hasattr must be called
@@ -267,23 +277,77 @@ class ClassGuiBase(BaseGui):
                     _hist.append((name, str(type(attr)), type(widget).__name__))
 
             except Exception as e:
-                hist_str = (
-                    "\n\t".join(map(lambda x: f"{x[0]} {x[1]} -> {x[2]}", _hist))
-                    + f"\n\t\t{name} ({type(attr)}) <--- Error"
-                )
-                if not hist_str.startswith("\n\t"):
-                    hist_str = "\n\t" + hist_str
-                if isinstance(e, MagicClassConstructionError):
-                    e.args = (f"\n{hist_str}\n{e}",)
-                    raise e
-                else:
-                    raise MagicClassConstructionError(
-                        f"\n{hist_str}\n\n{type(e).__name__}: {e}"
-                    ) from e
+                format_error(e, _hist, name, attr)
 
         self._unify_label_widths()
         return None
 
+    def _fast_insert(self, key: int, obj: Widget | Callable) -> None:
+        if isinstance(obj, Callable):
+            # Sometimes uses want to dynamically add new functions to GUI.
+            if isinstance(obj, FunctionGui):
+                if obj.parent is None:
+                    f = nested_function_gui_callback(self, obj)
+                    obj.called.connect(f)
+                widget = obj
+            else:
+                from ._base import _inject_recorder
+
+                obj = _inject_recorder(obj, is_method=False).__get__(self)
+                widget = self._create_widget_from_method(obj)
+
+            method_name = getattr(obj, "__name__", None)
+            if method_name and not hasattr(self, method_name):
+                object.__setattr__(self, method_name, obj)
+        else:
+            widget = obj
+
+        # _hide_labels should not contain Container because some ValueWidget like widgets
+        # are Containers.
+        _hide_labels = (
+            _LabeledWidget,
+            ButtonWidget,
+            ClassGuiBase,
+            FreeWidget,
+            Label,
+            Image,
+            Table,
+            FunctionGui,
+        )
+
+        if isinstance(widget, (ValueWidget, ContainerWidget)):
+            widget.changed.connect(lambda: self.changed.emit(self))
+
+        if hasattr(widget, _MCLS_PAREMT) or hasattr(widget.__class__, _MCLS_PAREMT):
+            widget.__magicclass_parent__ = self
+            if isinstance(widget, ClassGuiBase):
+                if self._remove_child_margins:
+                    widget.margins = (0, 0, 0, 0)
+                if (
+                    len(self.__magicclass_children__) > 0
+                    and widget is not self.__magicclass_children__[-1]
+                ):
+                    # nested magic classes are already in the list
+                    self.__magicclass_children__.append(widget)
+                    widget._my_symbol = Symbol(widget.name)
+
+                # NOTE: This is not safe. Attributes could collision and macro recording
+                # may break if not correctly named.
+
+        _widget = widget
+
+        if self.labels:
+            # no labels for button widgets (push buttons, checkboxes, have their own)
+            if not isinstance(widget, _hide_labels):
+                _widget = _LabeledWidget(widget)
+                widget.label_changed.connect(self._unify_label_widths)
+
+        self._list.insert(key, widget)
+        if key < 0:
+            key += len(self)
+        self._widget._mgui_insert_widget(key, _widget)
+        return None
+
 
 _C = TypeVar("_C", bound=ContainerWidget)
 
@@ -347,113 +411,12 @@ def make_gui(container: type[_C], no_margin: bool = True) -> type[_C | ClassGuiB
             else:
                 object.__setattr__(self, name, value)
 
-        def _fast_insert(self: cls, key: int, obj: Widget | Callable) -> None:
-            if isinstance(obj, Callable):
-                # Sometimes uses want to dynamically add new functions to GUI.
-                if isinstance(obj, FunctionGui):
-                    if obj.parent is None:
-                        f = nested_function_gui_callback(self, obj)
-                        obj.called.connect(f)
-                    widget = obj
-                else:
-                    obj = _inject_recorder(obj, is_method=False).__get__(self)
-                    widget = self._create_widget_from_method(obj)
-
-                method_name = getattr(obj, "__name__", None)
-                if method_name and not hasattr(self, method_name):
-                    object.__setattr__(self, method_name, obj)
-            else:
-                widget = obj
-
-            # _hide_labels should not contain Container because some ValueWidget like widgets
-            # are Containers.
-            _hide_labels = (
-                _LabeledWidget,
-                ButtonWidget,
-                ClassGuiBase,
-                FreeWidget,
-                Label,
-                Image,
-                Table,
-                FunctionGui,
-            )
-
-            if isinstance(widget, (ValueWidget, ContainerWidget)):
-                widget.changed.connect(lambda: self.changed.emit(self))
-
-            if hasattr(widget, "__magicclass_parent__") or hasattr(
-                widget.__class__, "__magicclass_parent__"
-            ):
-                widget.__magicclass_parent__ = self
-                if isinstance(widget, ClassGuiBase):
-                    if self._remove_child_margins:
-                        widget.margins = (0, 0, 0, 0)
-                    if (
-                        len(self.__magicclass_children__) > 0
-                        and widget is not self.__magicclass_children__[-1]
-                    ):
-                        # nested magic classes are already in the list
-                        self.__magicclass_children__.append(widget)
-                        widget._my_symbol = Symbol(widget.name)
-
-                    # NOTE: This is not safe. Attributes could collision and macro recording
-                    # may break if not correctly named.
-
-            _widget = widget
-
-            if self.labels:
-                # no labels for button widgets (push buttons, checkboxes, have their own)
-                if not isinstance(widget, _hide_labels):
-                    _widget = _LabeledWidget(widget)
-                    widget.label_changed.connect(self._unify_label_widths)
-
-            self._list.insert(key, widget)
-            if key < 0:
-                key += len(self)
-            self._widget._mgui_insert_widget(key, _widget)
-
-        def insert(self: cls, key: int, widget: Widget):
+        def insert(self: cls, key: int, widget: Widget) -> None:
             self._fast_insert(key, widget)
             self._unify_label_widths()
-
-        def show(self: cls, run: bool = True) -> None:
-            """
-            Show ClassGui. If any of the parent ClassGui is a dock widget in napari, then this
-            will also show up as a dock widget (floating if in popup mode).
-
-            Parameters
-            ----------
-            run : bool, default is True
-                *Unlike magicgui, this parameter should always be True* unless you want to close
-                the window immediately. If true, application gets executed *if needed*.
-            """
-            if self.__magicclass_parent__ is not None and self.parent is None:
-                # If child magic class is closed before, we have to set parent again.
-                self.native.setParent(
-                    self.__magicclass_parent__.native, self.native.windowFlags()
-                )
-
-            viewer = self.parent_viewer
-            if viewer is not None and self.parent is not None:
-                name = self.parent.objectName()
-                if name in viewer.window._dock_widgets:
-                    viewer.window._dock_widgets[name].show()
-                else:
-                    dock = viewer.window.add_dock_widget(
-                        self,
-                        name=self.name.replace("_", " ").strip(),
-                        area="right",
-                        allowed_areas=["left", "right"],
-                    )
-                    dock.setFloating(self._popup_mode == PopUpMode.popup)
-            else:
-                container.show(self, run=False)
-                self.native.activateWindow()
-                if run:
-                    run_app()
             return None
 
-        def reset_choices(self: cls, *_: Any):
+        def reset_choices(self, *_: Any):
             """Reset child Categorical widgets"""
             all_widgets: set[Widget] = set()
 
@@ -466,8 +429,53 @@ def make_gui(container: type[_C], no_margin: bool = True) -> type[_C | ClassGuiB
             for w in all_widgets:
                 if hasattr(w, "reset_choices"):
                     w.reset_choices()
+            return None
+
+        def show(self: cls, run: bool = True) -> None:
+            """
+            Show GUI. If any of the parent GUI is a dock widget in napari, then this
+            will also show up as a dock widget (floating if in popup mode).
+
+            Parameters
+            ----------
+            run : bool, default is True
+                *Unlike magicgui, this parameter should always be True* unless you want
+                to close the window immediately. If true, application gets executed if
+                needed.
+            """
+            if self.__magicclass_parent__ is not None and self.parent is None:
+                # If child magic class is closed before, we have to set parent again.
+                self.native.setParent(
+                    self.__magicclass_parent__.native, self.native.windowFlags()
+                )
+
+            viewer = self.parent_viewer
+            if viewer is not None and self.parent is not None:
+                name = self.parent.objectName()
+                if name in viewer.window._dock_widgets and isinstance(
+                    self.parent, QDockWidget
+                ):
+                    viewer.window._dock_widgets[name].show()
+                else:
+                    _floating = self._popup_mode == PopUpMode.popup
+                    _area = "left" if _floating else "right"
+                    dock = viewer.window.add_dock_widget(
+                        self,
+                        name=self.name.replace("_", " ").strip(),
+                        area=_area,
+                        allowed_areas=["left", "right"],
+                    )
+                    dock.setFloating(_floating)
+            else:
+                container.show(self, run=False)
+                self.native.activateWindow()
+                if run:
+                    run_app()
+            return None
 
         def close(self: cls):
+            """Close GUI. if this widget is a dock widget, then also close it."""
+
             current_self = self._search_parent_magicclass()
 
             viewer = current_self.parent_viewer
@@ -513,6 +521,8 @@ def make_gui(container: type[_C], no_margin: bool = True) -> type[_C | ClassGuiB
                 allowed_areas : sequence of str, optional
                     Allowed dock widget area. Allow all areas by default.
                 """
+                from ._dock_widget import QtDockWidget
+
                 name = name or widget.name
                 mainwin: QMainWindow = self.native
                 dock = QtDockWidget(
@@ -531,6 +541,8 @@ def make_gui(container: type[_C], no_margin: bool = True) -> type[_C | ClassGuiB
 
             @nogui
             def remove_dock_widget(self: cls, widget: Widget):
+                from ._dock_widget import QtDockWidget
+
                 dock = None
                 i_dock = -1
                 for i, child in enumerate(self.__magicclass_children__):
@@ -553,7 +565,6 @@ def make_gui(container: type[_C], no_margin: bool = True) -> type[_C | ClassGuiB
 
         cls.__init__ = __init__
         cls.__setattr__ = __setattr__
-        cls._fast_insert = _fast_insert
         cls.insert = insert
         cls.show = show
         cls.reset_choices = reset_choices
