@@ -4,8 +4,9 @@ import inspect
 from typing import Callable, Iterable, Iterator, Union, TYPE_CHECKING, TypeVar, overload
 from typing_extensions import ParamSpec
 import warnings
-from .utils import show_messagebox
+from magicgui.widgets import FunctionGui
 
+from .utils import show_messagebox
 from .types import Color
 from .signature import upgrade_signature
 
@@ -255,6 +256,7 @@ class Canceled(RuntimeError):
     """Raised when a function is canceled"""
 
 
+# TODO: confirm is not exchangable with thread_worker
 @overload
 def confirm(
     *,
@@ -356,7 +358,7 @@ def confirm(
         return _method
 
     if f is not None:
-        _decorator(f)
+        return _decorator(f)
     return _decorator
 
 
@@ -366,6 +368,90 @@ def nogui(method: F) -> F:
     return method
 
 
+def mark_preview(function: Callable, text: str = "Preview") -> Callable[[F], F]:
+    """
+    Define a preview of a function.
+
+    This decorator is useful for advanced magicgui creation. A "Preview" button
+    appears in the bottom of the widget built from the input function and the
+    decorated function will be called with the same arguments.
+    Following example shows how to define a previewer that prints the content of
+    the selected file.
+
+    .. code-block:: python
+
+        def func(self, path: Path):
+            ...
+
+        @mark_preview(func)
+        def _func_prev(self, path: Path):
+            with open(path, mode="r") as f:
+                print(f.read())
+
+    Parameters
+    ----------
+    function : callable
+        To which function previewer will be defined.
+    text : str, optional
+        Text of preview button.
+    """
+
+    def _wrapper(preview: F) -> F:
+        sig_preview = inspect.signature(preview)
+        sig_func = inspect.signature(function)
+        params_preview = sig_preview.parameters
+        params_func = sig_func.parameters
+
+        less = len(params_func) - len(params_preview)
+        if less == 0:
+            if params_preview.keys() != params_func.keys():
+                raise TypeError(
+                    f"Arguments mismatch between {sig_preview!r} and {sig_func!r}."
+                )
+            # If argument names are identical, input arguments don't have to be filtered.
+            _filter = lambda a: a
+
+        elif less > 0:
+            idx: list[int] = []
+            for i, param in enumerate(params_func.keys()):
+                if param in params_preview:
+                    idx.append(i)
+            # If argument names are not identical, input arguments have to be filtered so
+            # that arguments match the inputs.
+            _filter = lambda _args: (a for i, a in enumerate(_args) if i in idx)
+
+        else:
+            raise TypeError(
+                f"Number of arguments of preview function {preview!r} must be equal"
+                f"or smaller than that of running function {function!r}."
+            )
+
+        def _preview(*args):
+            # find proper parent instance in the case of classes being nested
+            from .gui import BaseGui
+
+            if len(args) > 0 and isinstance(args[0], BaseGui):
+                ins = args[0]
+                prev_ns = preview.__qualname__.split(".")[-2]
+                while ins.__class__.__name__ != prev_ns:
+                    ins = ins.__magicclass_parent__
+                args = (ins,) + args[1:]
+            # filter input arguments
+            return preview(*_filter(args))
+
+        if not isinstance(function, FunctionGui):
+            upgrade_signature(
+                function, additional_options={"preview": (text, _preview)}
+            )
+        else:
+            from .gui._function_gui import append_preview
+
+            append_preview(function, _preview, text=text)
+        return preview
+
+    return _wrapper
+
+
 def _assert_iterable(obj):
     if obj is None:
         obj = []
